#include "defines.h"
#include "kozos.h"
#include "task_display.h"
#include "lib.h"
#include "sw.h"
#include "re.h"
#include "intr.h"

#define H8_3069F_ISCR       ((volatile uint8 *)0xFEE014)
#define H8_3069F_IER        ((volatile uint8 *)0xFEE015)
#define H8_3069F_ISR        ((volatile uint8 *)0xFEE016)

#define H8_3069F_TSTR       ((volatile uint8 *)0xFFFF60)
#define H8_3069F_TISRC      ((volatile uint8 *)0xFFFF66)
#define H8_3069F_16TCR0     ((volatile uint8 *)0xFFFF68)
#define H8_3069F_TIOR0      ((volatile uint8 *)0xFFFF69)
#define H8_3069F_16TCNT0H   ((volatile uint8 *)0xFFFF6A)
#define H8_3069F_16TCNT0L   ((volatile uint8 *)0xFFFF6B)

#define MAX_BIT_COUNT (64)

#define TCNT_COUNT_START()  (*H8_3069F_TSTR |= (1 << 0))
#define TCNT_COUNT_STOP()   (*H8_3069F_TSTR &= ~(1 << 0))
#define TCNT_COUNT_VALUE()  (((*H8_3069F_16TCNT0H) << 8) | ((*H8_3069F_16TCNT0L) << 0))
#define TCNT_COUNT_RESET()  \
  do { \
    *H8_3069F_16TCNT0H = 0; \
    *H8_3069F_16TCNT0L = 0; \
  } while (0)

#define TCR0_SETUP()        (*H8_3069F_16TCR0 = 0x03)
#define TMRINT_ENABLE()     (*H8_3069F_TISRC |= (1 << 4))
#define TMRINT_FLAG_CLEAR() (*H8_3069F_TISRC &= ~(1 << 0))
#define IRQ4_ENABLE() \
  do { \
    (*H8_3069F_ISCR |= (1 << 4)); \
    (*H8_3069F_IER |= (1 << 4)); \
  } while (0)

typedef enum {
  WaitLeader,
  FoundLeader,
  Receiving,
  Received
} ir_state_t;

static uint16 bitcnt = 0;
static uint16 tmrovf = 0;
static uint8 data[MAX_BIT_COUNT / 8];
static ir_state_t irs = WaitLeader;

static void remocon_intr_edge(void)
{
  switch (irs) {
    case WaitLeader:
      TCNT_COUNT_RESET();
      TCNT_COUNT_START();
      bitcnt = 0;
      tmrovf = 0;
      irs = FoundLeader;
      break;
    case FoundLeader:
      irs = Receiving;
      break;
    case Receiving:
      if (bitcnt < MAX_BIT_COUNT) {
        if ((bitcnt % 8) == 0) {
          data[bitcnt / 8] = 0;
        }
        uint16 tmrval = TCNT_COUNT_VALUE();
        if (tmrval > 0x1077) {
          data[bitcnt / 8] |= (1 << (bitcnt % 8));
        }
        bitcnt++;
      }
      if (32 <= bitcnt) {
        irs = Received;
      }
      break;
    case Received:
      TCNT_COUNT_STOP();
      break;
    default:
      TCNT_COUNT_STOP();
      break;
  }
  TCNT_COUNT_RESET();
}

static void remocon_intr_tovf(void)
{
  TMRINT_FLAG_CLEAR();
  tmrovf = 1;
  irs = WaitLeader;
  TCNT_COUNT_STOP();
}

int task_input(int argc, char *argv[])
{
  int nx = 40, ny = 12;

  /*
   * Setup.
   */
  kz_setintr(SOFTVEC_TYPE_IR_EDGE, remocon_intr_edge);
  kz_setintr(SOFTVEC_TYPE_IR_TOVF, remocon_intr_tovf);
  TCR0_SETUP();
  TMRINT_ENABLE();
  IRQ4_ENABLE();

  /*
   * Display clear
   */
  display_clear();
  display_draw_logo(  0, 0, 2);
  display_draw_box(0, 0, 121, 31, 1);
  display_draw_text(40, 4, "KOZOS EXPBRD #00");
  display_draw_logo(nx, ny, 0);

  uint16 prev = 0, curr = 0;
  while (1) {
    curr = re_read();
    if (curr != prev) {
      int dir = (int)(curr & 0xff) - (int)(prev & 0xff);
      if (dir < 0) {
        if (-32 < dir) {
          display_led_write(1, 0);
          display_led_toggle(0);
        }
        if (40 < nx) {
          nx--;
        }
      } else {
        if (dir < 32) {
          display_led_write(0, 0);
          display_led_toggle(1);
        }
        if (nx < 100) {
          nx++;
        }
      }
      prev = curr;

      display_draw_box(nx + 20, ny, nx + 20, ny + 16, 0);
      display_draw_logo(nx, ny, 0);
      display_led_toggle(3);
    }

    if (tmrovf) {
      if (bitcnt == 32) {
#if 0
        int i;
        for (i = 0; i < bitcnt; i++) {
          if (data[i / 8] & (1 << (i % 8))) {
            console_write("1");
          } else {
            console_write("0");
          }
        }
        console_write("\n");
#endif
        audio_pulse();
        display_led_toggle(2);
      }
      bitcnt = 0;
      tmrovf = 0;
    }
    kz_wait();
  }

  return 0;
}

